
/*
 *	KDE3 dotCURVE Style ( version 1.0rc1 )
 *	Copyright ( C ) 2004, Matias Costa <mcosta@onexige.com>
 *	
 *	Many widget code from Konx:
 *	KDE3 Konx Style ( version 0.1 )
 *	Copyright ( C ) 2004, Martin Stubenschrott <stubenschrott@gmx.net>
 *
 *	Based on dotNET:
 *	Copyright ( C ) 2001-2002, Chris Lee <lee@azsites.com>
 *				 Carsten Pfeiffer <pfeiffer@kde.org>
 *				 Karol Szwed <gallium@kde.org>
 *
 *	Drawing routines completely reimplemented from KDE3 HighColor, which was
 *	originally based on some stuff from the KDE2 HighColor.
 *
 *	This library is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU Library General Public
 *	License version 2 as published by the Free Software Foundation.
 *
 *	This library is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	Library General Public License for more details.
 *
 *	You should have received a copy of the GNU Library General Public License
 *	along with this library; see the file COPYING.LIB. If not, write to
 *	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *	Boston, MA 02111-1307, USA.
 */

#include <qstyleplugin.h>
#include <qstylefactory.h>
#include <qpointarray.h>
#include <qpainter.h>
#include <qtabbar.h>
#include <qcombobox.h>
//#include <qscrollbar.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
//#include <qtoolbar.h>
//#include <qmenubar.h>
#include <qpopupmenu.h>
//#include <qdrawutil.h>
#include <qapplication.h>
//#include <qvariant.h>
//#include <qpixmapcache.h>
#include <qprogressbar.h>
#include <qslider.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qtabwidget.h>
#include <qheader.h>
#include <kpixmap.h>
#include <qsettings.h>

#include "dotcurve.h"
#include "dotcurve.moc"

// The order for the arrows is: up, down, left, right
#define UP_ARROW 0
#define DOWN_ARROW 1
#define LEFT_ARROW 2
#define RIGHT_ARROW 3

// The vertical shift of a selected tab (up or down)
#define TAB_SHIFT 4

#define PEN( cg ) ( customPen ? customPenColor : cg.background().dark() )
#define MENU_BRUSH2( cg ) (cg.background().dark( 120 ))
#define SB_BGCOLOR( cg ) ( useCustomScrollbarColor? customScrollbarColor : cg.base() )

// This is for helping me to develop the style while
// kdm, my brother and myself uses the style
// if DEVEL defined the factory will ask for "test"
// so I can do "kwhatever --style test"
//#define DEVEL 1
//#include <kdebug.h>

static bool staticDefined = false;
static QPointArray arrowPoints[4];

static bool drawToolBarSeparator;
static bool centerTabs;
static bool statusbarSeparator;
static bool useCustomScrollbarColor;
static QColor customScrollbarColor;
static bool customPen;
static QColor customPenColor;
static QColor checkColor;
static QColor radioColor;
static QString scrollBarStyle;
static int tabOverlap = 7;

static bool translucentMenus = false;

class MyTabWidget : public QTabWidget 
{
public:
	const QTabWidget *tw;
	MyTabWidget(const QTabWidget *w) {tw = w;}
	~MyTabWidget() {}
	
	QTabBar* myTabBar() { return tabBar(); }
};

// -- Style Plugin Interface -------------------------
class dotCURVEstylePlugin : public QStylePlugin
{
public:
	dotCURVEstylePlugin() {}
	~dotCURVEstylePlugin() {}

	QStringList keys() const
	{
		#ifndef DEVEL
		return QStringList() << "dotCURVE";
		#else
		return QStringList() << "test";
		#endif
	}

	QStyle* create( const QString& key )
	{
		
		#ifndef DEVEL
		if ( key == "dotcurve" )
		#else
		if ( key == "test" )
		#endif
			return new dotCURVEstyle;
		return 0;
	}
};

Q_EXPORT_PLUGIN( dotCURVEstylePlugin )

// ---------------------------------------------------

dotCURVEstyle::dotCURVEstyle() : KStyle( AllowMenuTransparency ,ThreeButtonScrollBar ), 
khtml( false )
{
	if ( !staticDefined ) {
		arrowPoints[0].setPoints( 7, -4,1, 2,1, -3,0, 1,0, -2,-1, 0,-1, -1,-2 );
		arrowPoints[1].setPoints( 7, -4,-2, 2,-2, -3,-1, 1,-1, -2,0, 0,0, -1,1 );
		arrowPoints[2].setPoints( 7, 0,-3, 0,3, -1,-2, -1,2, -2,-1, -2,1, -3,0 );
		arrowPoints[3].setPoints( 7, -2,-3, -2,3, -1,-2, -1,2, 0,-1, 0,1, 1,0 );
		
		staticDefined = true;
	}
	
	QSettings settings;
	settings.beginGroup( "/dotcurvestyle/Settings" );
	drawToolBarSeparator = settings.readBoolEntry( "drawToolBarSeparator", true );
	centerTabs = settings.readBoolEntry("centerTabs", true);
	customPen = settings.readBoolEntry( "useCustomBrushColor", false );
	statusbarSeparator = settings.readBoolEntry( "statusbarSeparator", true );
	useCustomScrollbarColor = settings.readBoolEntry( "useCustomScrollbarColor", false );
	customScrollbarColor = settings.readNumEntry( "customScrollbarColor", (int)white.rgb() );
	customPenColor = settings.readNumEntry( "customBrushColor", (int)black.rgb() );
	// -32768 is orange
	checkColor = settings.readNumEntry( "checkColor", -32768 );
	radioColor = settings.readNumEntry( "radioColor", -32768 );
	scrollBarStyle = settings.readEntry( "scrollBarStyle", "ThreeButtonScrollBar" );
	tabOverlap = settings.readNumEntry( "tabOverlap", 7 );
	settings.endGroup();
	
	// safe check
	if ( tabOverlap < 0 ) tabOverlap = 0;
	
	if( !strcmp( scrollBarStyle, "WindowsStyleScrollBar" ) )
		this->setScrollBarType( KStyle::WindowsStyleScrollBar );
	else if( !strcmp( scrollBarStyle, "PlatinumStyleScrollBar" ) )
		this->setScrollBarType( KStyle::PlatinumStyleScrollBar );
	else if( !strcmp( scrollBarStyle, "NextStyleScrollBar" ) )
		this->setScrollBarType( KStyle::NextStyleScrollBar );
	
}


dotCURVEstyle::~dotCURVEstyle()
{
}

/*
 sharpCorner tells what corner(s) paint sharp ( surprised? )
 Corner is a defined by Qt ( this people is really kind )
 but there is a value == 0, so we can not tell we want
 all 4 corners round. Also I define some convenience values
 as none, top... etc
*/
enum MyCorner { C_None = 0x0, 
	C_TopLeft = 0x01, C_BottomLeft = 0x02, C_BottomRight = 0x04, C_TopRight = 0x08, 
	C_Top = 0x09,
	C_Bottom = 0x06,
};

void renderRoundRect( QPainter* p,
			const int x, const int y, const int w, const int h,
			uint sharpCorner = C_None ) 
{
	const int x2 = x + w -1, y2 = y + h -1;
	int i;

	QPointArray pa(12);
	// We define points in anti-clockwise order, starting in top-left
	i=1;
	if ( sharpCorner & C_TopLeft ) pa.setPoint( 0, x,y );
	else {
		
		pa.putPoints(0, 3, x+5,y, x+2,y+2, x,y+5 );
		i=3;
	}
	if ( sharpCorner & C_BottomLeft ) pa.setPoint( i++, x,y2 );
		else {
		pa.putPoints( i, 3, x,y2-5, x+2,y2-2, x+5,y2 );
		i+=3;
	}
	if ( sharpCorner & C_BottomRight ) pa.setPoint( i++, x2,y2 );
	else {
		pa.putPoints( i, 3, x2-5,y2, x2-2,y2-2, x2,y2-5 );
		i+=3;
	}
	if ( sharpCorner & C_TopRight ) pa.setPoint( i++, x2,y );
	else {
		pa.putPoints( i, 3, x2,y+5, x2-2,y+2, x2-5,y );
		i+=3;
	}

	p->drawConvexPolygon( pa, 0, i );
}

// Taken the look from konx
void renderGripper( QPainter *p,
			int x, int y, int w, int h,
			bool horizontal = false )
{
	int cx = x + ( w/2 );
	int cy = y + ( h/2 );
	
	if ( horizontal ){
		// adjust length to keep centered
		const int l = 7 + ( h%2 );
		cy -= 4;
		// draw gripper
		if ( w >= 18 ) {
			p->drawLine( cx-6, cy  , cx-6, cy+l   );
			p->drawLine( cx-3, cy+1, cx-3, cy+l-1 );
			p->drawLine( cx  , cy ,  cx , cy+l    );
			p->drawLine( cx+3, cy+1, cx+3, cy+l-1 );
			p->drawLine( cx+6, cy  , cx+6, cy+l   );

		} else if ( w >= 11 ) {
			p->drawLine( cx-3, cy  , cx-3, cy+l   );
			p->drawLine( cx  , cy+1, cx  , cy+l-1 );
			p->drawLine( cx+3, cy  , cx+3, cy+l   );
		}
	} else {// vertical
		// adjust length to keep centered
		const int l = 7 +( w%2 );
		cx -= 4;
		// draw gripper
		if ( h >= 18 ) {
			p->drawLine( cx  , cy-6, cx+l  , cy-6 );
			p->drawLine( cx+1, cy-3, cx+l-1, cy-3 );
			p->drawLine( cx  , cy  , cx+l  , cy   );
			p->drawLine( cx+1, cy+3, cx+l-1, cy+3 );
			p->drawLine( cx  , cy+6, cx+l  , cy+6 );
		} else if ( h >= 11 ) {
			p->drawLine( cx  , cy-3, cx+l  , cy-3 );
			p->drawLine( cx+1, cy  , cx+l-1, cy   );
			p->drawLine( cx  , cy+3, cx+l  , cy+3 );
		}
	}
}

void dotCURVEstyle::renderMenuBlendPixmap( KPixmap &pix, const QColorGroup &cg, const QPopupMenu */*popup*/ ) const
{
	QPainter p( &pix );
	pix.fill( cg.background() );
	translucentMenus = true;
	//kdDebug() << "renderMenuBlendPixmap: translucentMenus <- true\n";
}

void dotCURVEstyle::renderButton(QPainter *p,
				 const QRect &r,
				 const QColorGroup &cg,
				 bool sunken) const
{
	int x, y, w, h;
	r.rect(&x,&y,&w,&h);

//        p->save();
	if (sunken) {
		p->setPen(cg.highlight().dark());
		p->setBrush(cg.highlight());
	} else {
		p->setPen(customPen? customPenColor : cg.button().dark());
		p->setBrush(cg.button());
	}
	if(( w < 16 ) || ( h < 16 ))
		p->drawRect(x, y, w, h);
	else
		renderRoundRect(p, x, y, w, h);
//        p->restore();
}

void dotCURVEstyle::drawKStylePrimitive( KStylePrimitive kpe,
		QPainter* p,
		const QWidget* widget,
		const QRect &r,
		const QColorGroup &cg,
		SFlags flags,
		const QStyleOption& opt ) const
{
	int x, y, w, h;
	r.rect( &x, &y, &w, &h );
	// SLIDER
	// ------
	switch( kpe ) {
	case KPE_SliderGroove: {
		const QSlider* slider = ( const QSlider* )widget;
		const bool horizontal = slider->orientation() == Horizontal;
		int gcenter;

		p->setPen( QPen( PEN( cg ), 2 ) );
		if ( horizontal ) {
			gcenter = y + h/2;
			p->drawLine( x, gcenter, x+w, gcenter );
		} else {
			gcenter = x + w/2;
			p->drawLine( gcenter, y, gcenter, y + h );
		}
		break;
	}

	case KPE_SliderHandle: {
		const QSlider* slider = ( const QSlider* )widget;
		const bool horizontal = slider->orientation() == Horizontal;
		renderButton( p, r, cg, flags & Style_Active );
		if ( flags & Style_Active ) p->setPen( cg.highlightedText() );
		renderGripper( p, x, y, w, h, !horizontal );
		break;
	}

	default:
		KStyle::drawKStylePrimitive( kpe, p, widget, r, cg, flags, opt );
	}
}

// This function draws primitive elements as well as their masks.
void dotCURVEstyle::drawPrimitive( PrimitiveElement pe,
				QPainter *p,
				const QRect &r,
				const QColorGroup &cg,
				SFlags flags,
				const QStyleOption &opt ) const
{
	const bool down = flags & Style_Down;
	const bool on = flags & Style_On;
//	bool sunken = flags & Style_Sunken;
	const bool horiz = flags & Style_Horizontal;
	const bool focused = flags & Style_HasFocus;
	bool enabled = flags & Style_Enabled;
	//const bool mouseover = flags & Style_MouseOver;
	int x, y, w, h, x2, y2;
	int s = 0;
	r.rect( &x, &y, &w, &h );
	x2 = x + w -1;
	y2 = y + h -1;
	switch( pe ) {

	//	BUTTONS
	//	-------
	case PE_HeaderSection:
	case PE_ScrollBarSlider:
		if ( !kickerMode ) cleanSides( p, x, y, w, h, SB_BGCOLOR( cg ), horiz );
	case PE_ButtonBevel:
	case PE_ButtonTool:
	case PE_ButtonDropDown:
	case PE_ButtonDefault:
	case PE_ButtonCommand: {
		renderButton( p, r, cg, on || down );
		if ( pe == PE_ScrollBarSlider ) {
			if ( on || down ) p->setPen( cg.highlightedText() );
			renderGripper( p, x, y, w, h, horiz );
		} else if ( pe == PE_ButtonDefault ) {
			if ( on || down ) p->setPen( QPen(cg.highlight().dark(), 2) );
			else p->setPen( QPen(cg.highlight(), 2) );
			p->setBrush(NoBrush);
			renderRoundRect(p, x, y , w, h);
			p->drawLine(x + 1, y + 6, x  + 1, y2 - 5);
			p->drawLine(x + 5, y + 1, x2 - 5, y  + 1);                        
			break;
		}

		break;
	}
	case PE_ScrollBarAddPage:
	case PE_ScrollBarSubPage: {
		const QColor bgColor = SB_BGCOLOR( cg );
		if ( on || down ) p->fillRect( r, bgColor.dark() );
		else p->fillRect( r, bgColor );
		break;
	}


//	GENERAL PANELS
//	--------------
	case PE_Splitter:{
		p->setPen( QPen( PEN( cg ), 2, DotLine ) );
		const int xCenter = x + w/2, yCenter = y + h/2;
		if ( horiz ) p->drawLine( xCenter, yCenter - 12, xCenter, yCenter + 12 );
		else p->drawLine( xCenter - 12, yCenter, xCenter +12, yCenter );
		break;
	}
	case PE_PanelLineEdit: {
		
		QLineEdit *mySelfLineEdit = dynamic_cast<QLineEdit*>( p->device() );
		if ( mySelfLineEdit ) enabled = !mySelfLineEdit->isReadOnly() && enabled;
		p->setBrush( enabled? cg.base() : cg.background() );
		p->setPen( focused? cg.highlight() : PEN( cg ) );
		if ( khtml ) p->drawRect( x, y, w, h );
		else {
			cleanSides( p, x, y, w, h, cg.background() );
			renderRoundRect( p, x, y, w, h );
		}
		break;
	}
	case PE_Panel: {
		// HACK: In kicker do not paint frames
		if ( kickerMode ) return;
		if ( ( w < 3 ) || ( h < 3 ) ) return;
		bool filling = false;
		p->setBrush( NoBrush );
		QWidget *mySelf = dynamic_cast<QWidget*>( p->device() );
		if ( mySelf ) {
			//kdDebug() << "PE_Panel: " << mySelf << "\n";
			if (   mySelf->inherits( "KColorCells" )
			    || mySelf->inherits( "KAnimWidget" ) ) return;
			const QWidget *firstChild = mySelf->childAt( x + 5, y + 5 );
			if ( firstChild ) {
 				// BIG HACK: QClipperWidget paletteBackgroundColor returns #555552. Why?
				if (   !strcmp( firstChild->name(), "qt_viewport" ) 
				    || !strcmp( firstChild->name(), "qt_clipped_viewport" ) )
					p->setBrush( cg.base() );
				else p->setBrush( firstChild->paletteBackgroundColor() );
				filling = true;
			}
		}

		// Here we do not paint into the panel to avoid flickering
		if ( filling ) {
			QRegion region( x, y, w, h );
			p->setClipRegion( region.subtract( QRegion( x +3, y +3, w -6, h -6 ) ) );
			p->setClipping( true );
		}
		p->setPen( PEN( cg ) );
		renderRoundRect( p, x, y, w, h );
		if ( filling ) p->setClipping( false );
		break;
		
	}
	case PE_TabBarBase: 
		//kdDebug() << "PE_TabBarBase: " << w << "\n";
		p->fillRect(r, green);
		break;

	case PE_PanelPopup: {
		//kdDebug() << "PE_PanelPopup: translucentMenus == "<< translucentMenus << "\n";
		p->setPen( MENU_BRUSH2( cg ) );
		if ( translucentMenus ) p->setBrush( NoBrush );
		else p->setBrush( cg.background() );
		renderRoundRect( p, x, y, w, h, C_TopLeft );
		if ( !translucentMenus ) {
			p->setClipRect( QRect( x, y, 25, h ) );
			p->setBrush( MENU_BRUSH2( cg ) );
			renderRoundRect( p, x, y, 25 + ROUNDCORNERD /2, h, C_TopLeft );
			p->setClipping( false );
		}
		break;
	}	

	case PE_PanelGroupBox:{
		p->setPen( PEN( cg ) );
		p->setBrush( NoBrush );
		renderRoundRect( p, x, y, w, h );
		break;
	}
	case PE_PanelTabWidget: {
		// HACK: I don't know why I have to touch this
		++x; --w;
		uint sharp = C_Top;
		const QWidget *qw = dynamic_cast<const QWidget*>( p->device() );		
		if ( qw && qw->parentWidget() ) {
			const QTabWidget *tw 
				= dynamic_cast<const QTabWidget*>( qw->parentWidget() );
			if ( tw && tw->tabPosition() == QTabWidget::Bottom ) 
				sharp = C_Bottom;
		}
		
		p->setPen( PEN( cg ) );
		p->setBrush( NoBrush );
		renderRoundRect(p, x, y, w, h, sharp);
		break;
	}

//	MENU / TOOLBAR PANEL
//	--------------------
	case PE_PanelMenuBar:			 // Menu
	case PE_PanelDockWindow: {		// Toolbar
		p->fillRect( x, y, w, h, cg.background() );
		break;
	}

//	TOOLBAR/DOCK WINDOW HANDLE
//	--------------------------
	case PE_DockWindowResizeHandle: {
		renderButton( p, r, cg );
		renderGripper( p, x, y, w, h );
		break;
	}

	case PE_DockWindowHandle: {
		if ( !drawToolBarSeparator ) return;
		QPen dotPen( cg.mid(), 3, DotLine );
		p->setPen( dotPen );
		p->setBrush( NoBrush );
		if ( horiz ){
			p->drawLine( x + 1, y, x + 1, y2 - 3 );
			p->drawLine( x + 4, y + 3, x + 4, y2 );
		} else {
			p->drawLine( x, y + 1, x2 - 3, y + 1 );
			p->drawLine( x + 3, y + 4, x2, y + 4 );
		}
		break;
	}

//	TOOLBAR SEPARATOR
//	-----------------
	case PE_DockWindowSeparator: {
		if ( !drawToolBarSeparator ) return;
		// Calculation to get full dots:
		// The line long must be 2*n
		int l; 
		QPen dotPen( cg.mid(), 2, DotLine );
		p->setPen( dotPen );
		if ( horiz ){
			h -= 5;
			l = h - ( h % 2 );
			x += w/2;
			p->drawLine( x, y + 5, x, y + l );
		} else {
			w -= 5;
			l = w - ( w % 2 );
			y += h/2;
			p->drawLine( x + 5, y, x + l, y );
		}
		
		break;
	}
//	SCROLLBAR BUTTONS
//	-----------------
	case PE_ScrollBarAddLine:
		s++;
	case PE_ScrollBarSubLine:
		// I don't know if to guess the arrow this way it is faster
		// but it is much more fun
		if ( horiz ) s += 2;
		if ( down ) {
			p->setPen( cg.highlight() );
			p->setBrush( cg.highlight() );
			p->drawEllipse( x, y, w, h );
		} else p->fillRect( x, y, w, h, SB_BGCOLOR( cg ) );
		s -= 3;
	case PE_ArrowRight:
		s++;
	case PE_ArrowLeft:
		s++;
	case PE_SpinWidgetDown:
	case PE_ArrowDown:
		s++;
	case PE_HeaderArrow:
	case PE_SpinWidgetUp:
	case PE_ArrowUp: {
		// Special case for The header arrow. Why does not exist PE_HeaderArrowUp?
		if ( ( pe == PE_HeaderArrow ) && !( flags & Style_Up ) ) {
			 ++s;
			 p->setPen( cg.buttonText() );
		} else if ( down || ( flags & Style_Active ) ) p->setPen( cg.highlightedText() );
		else p->setPen( cg.buttonText() );

		QPointArray &a = arrowPoints[s];
		// Translate to te good place
		a.translate( x + w/2, y + h /2 );
		p->drawLineSegments( a, 0, 3 );
		p->drawPoint( a[6] );
		// Back the points to origin
		a.translate( -x - w/2, -y - h /2 );
		break;
	}

	case PE_FocusRect: {
		// Need a little tweak for negative width or heigth
		if ( h < 0 ) {
			y += h;
			h = -h;
		}
		if ( w < 0 ) {
			x += w;
			w = -w;
		}
		if ( ( w < 8 ) || ( h < 8 ) ) p->drawWinFocusRect( x, y, w, h );
		else {
			p->save();
			p->setPen( QPen( cg.highlight(), 0, DotLine ) );
			p->setBrush( NoBrush );
			renderRoundRect( p, x, y, w, h );
			p->restore();
		}
		break;
	}
	case PE_CheckMark: {
		const int cx = r.center().x() - 3, cy = r.center().y() - 3;
		if ( on || down ) {
			QPen pen;
			if ( enabled )
				if ( down ) pen.setColor( checkColor.light() );
				else pen.setColor( checkColor );
			else pen.setColor( cg.mid() );
			pen.setWidth ( 2 );
			p->setPen( pen );
			p->setBrush( down ? cg.highlight() : pen.color() );
			p->drawLine ( cx,cy,   cx+6,cy+6 );
			p->drawLine ( cx,cy+6, cx+6,cy   );
			// fix missing points at the end of the lines
			p->drawPoint ( cx+6, cy   );
			p->drawPoint ( cx+6, cy+6 );
		} else if ( flags & Style_Off ) {} // empty
		else {	// tristate
			p->setPen ( NoPen );
			p->drawRect ( cx,cy, cx+4,cy+4 );
		}
		break;
	}
	case PE_ExclusiveIndicator: {
		const int min = QMIN( w, h );
		p->setBrush( cg.base() );
		p->setPen( PEN( cg ) );
		p->drawEllipse( x + ( x - min )/2, y + ( y - min )/2, min, min );
		break;
	}

	case PE_Indicator: {
		p->setPen( PEN( cg ) );
		p->setBrush( flags & Style_Enabled ? cg.base() : cg.background() );
		p->drawRect( r );
		break;
	}
	case PE_Separator: {
		const int cy = h > 2? y + h/2 : y;
		p->setPen( PEN( cg ) );
		if ( w > 30 ) {
			const int cx = x + w/2; 
			p->drawLine( x, cy, cx - 10, cy );
			p->drawLine( cx - 5, cy, cx + 5, cy );
			p->drawLine( cx + 10, cy, x2, cy );
		} else p->drawLine( x, cy, x2, cy );
		break;
	}
	case PE_StatusBarSection: {
		if ( statusbarSeparator ){
			const int yCenter = y + h/2;
			p->setPen( QPen( PEN( cg ), 0, DotLine ) );
			p->drawLine( x2 + 2, yCenter - 7, x2 + 2, yCenter + 8 );
		}
		break;
	}
	default: KStyle::drawPrimitive( pe, p, r, cg, flags, opt );
	}
}


void dotCURVEstyle::drawControl( ControlElement element,
				QPainter *p,
				const QWidget *widget,
				const QRect &r,
				const QColorGroup &cg,
				SFlags flags,
				const QStyleOption& opt ) const
{
	const bool selected = flags & Style_Selected;
//	const bool enabled = flags & Style_Enabled;
	const bool active = flags & Style_Active;
	const bool focused = flags & Style_HasFocus;
	int x, y, w, h, x2, y2;

	r.rect( &x, &y, &w, &h );
	x2 = x + w -1;
	y2 = y + h -1;
	switch ( element ) {
	//	TABS
	//	----
	case CE_TabBarTab: {
		const QTabBar *tb = ( const QTabBar * ) widget;
		p->setPen( PEN( cg ) );
		if ( selected ) p->setBrush( cg.background() );
		else p->setBrush( cg.mid().light(105) );
		bool drawPoints = true;
		if ( const QTabWidget *tw = ::qt_cast<QTabWidget*>( tb->parent() ) ) {
			// is there a corner widget in the (top) left edge?
			QWidget *cw = tw->cornerWidget(Qt::TopLeft);
			if (cw) drawPoints = false;
		}

		/*
		QPoint corner(0, 0);
		kdDebug() << "tab corner: " << tb->mapFromParent( corner ).x() << "\n";
		*/
		switch ( tb->shape() ) {
		case QTabBar::RoundedAbove:
			h-=2; y2-=2;
			
			if ( tb->indexOf( opt.tab()->identifier() ) == 0 && drawPoints ) {
				p->drawPoint( x, y2 +1 );
				p->drawPoint( x, y2 +2 );
			}
			
			if ( selected ) {
				renderRoundRect( p, x, y, w, h, C_Bottom);
				p->setPen( cg.background() );
				p->drawLine( x +1, y2, x2 -1, y2 );

			}
			else if ( tb->indexOf( opt.tab()->identifier() ) == 0 ) {
				// First tab
				renderRoundRect(p, x, y + TAB_SHIFT, w, h - TAB_SHIFT, C_TopRight | C_Bottom );
			} else if (tb->indexOf(opt.tab()->identifier()) == tb->count() - 1) {
				// Last tab
				renderRoundRect(p, x, y + TAB_SHIFT, w, h - TAB_SHIFT, C_TopLeft | C_Bottom );
			} else {
				p->drawRect( x, y + TAB_SHIFT, w, h - TAB_SHIFT);
			}
			break;

		case QTabBar::RoundedBelow:
			// TODO: look at tab code seriously
			y+=3; h-=3; 
			if ( tb->indexOf( opt.tab()->identifier() ) == 0 && drawPoints ) {
				p->drawPoint( x, y -1 );
				p->drawPoint( x, y -2 );
				p->drawPoint( x, y -3 );
			}
			if ( selected ) {
				renderRoundRect( p, x, y, w, h, C_Top );
				p->setPen( cg.background() );
				p->drawLine( x +1, y, x2 -1, y);
			} else if ( tb->indexOf( opt.tab()->identifier() ) == 0 ) {
				// First tab
				renderRoundRect(p, x, y, w, h - TAB_SHIFT, C_BottomRight | C_Top );
			} else if ( tb->indexOf(opt.tab()->identifier()) == tb->count() - 1 ) {
				// Last tab
				renderRoundRect(p, x, y, w, h - TAB_SHIFT, C_BottomLeft | C_Top );
			} else {
				p->drawRect( x, y, w, h - TAB_SHIFT);
			}
			
			break;

		default: KStyle::drawControl( element, p, widget, r, cg, flags, opt );
		}
		break;
	}
	case CE_TabBarLabel: {
		const int text_flags = AlignVCenter | AlignHCenter | ShowPrefix |
				DontClip | SingleLine;
		const QTabBar * tb = ( const QTabBar * ) widget;

		if ( focused ) {
			int tx, ty, tw, th;
			opt.tab()->rect().rect( &tx, &ty, &tw, &th );
			p->save();
			p->setPen( QPen( cg.highlight(), 0, DotLine ) );
			p->setBrush( NoBrush );
			renderRoundRect( p, tx +3, ty +3, tw -6 , th -6);
			p->restore();
		}
		// FIXME: selected is always false here (??). This method
		// is not recomended in the qt docs, but I do not know
		// how to make it better
		const bool selectedTab = tb->currentTab() == opt.tab()->identifier();
		//kdDebug() << "CE_TabBarLabel: " << selected << " / " << selectedTab <<" \n";			
		switch ( tb->shape() ) {
		case QTabBar::TriangularAbove:
		case QTabBar::RoundedAbove:
			// icon-text align tune
			y += 4;
			h -= 4;
			if ( selectedTab ) y -= 2;
			break;
		default:
			// icon-text align tune
			y -= 4;
			h += 4;
			if ( selectedTab ) y += 2;
			break;
		}
		
		p->setPen( cg.foreground() );
		p->drawText( x, y, w, h, text_flags, opt.tab()->text() );
		break;
		
					
	}
	case CE_PushButton: {
		const QPushButton *button = (const QPushButton*)widget;
		// HACK for the close button in tabs
		if ( button->parentWidget() && button->parentWidget()->inherits( "QTabBar" ) ) {
			const bool down = flags & Style_Down;
			const bool on = flags & Style_On;
			p->setPen( PEN( cg ) );
			if ( on || down ) p->setBrush( cg.highlight() );
			else p->setBrush( cg.button() );
			p->drawRect( x, y, w, h );
			return;
		}
		
		const PrimitiveElement pe = button->isDefault() ? PE_ButtonDefault : PE_ButtonBevel;
		drawPrimitive( pe, p, r, cg, flags );
		break;
	}
	case CE_PushButtonLabel: {
		const QPushButton* button = static_cast<const QPushButton *>( widget );
		bool active = button->isOn() || button->isDown();
		// pixmap width or 0 if there is not
		int pixw = 0;
		// efective x and width. x == ex if there is no icon nor menu
		int ex = x, ew = w;

		// Draw the icon if there is one
		if ( button->iconSet() && !button->iconSet()->isNull() )
		{
			QIconSet::Mode mode = QIconSet::Disabled;
			QIconSet::State state = QIconSet::Off;

			if ( button->isEnabled() ) mode = button->hasFocus() ? QIconSet::Active : QIconSet::Normal;
			if ( button->isToggleButton() && button->isOn() ) state = QIconSet::On;

			QPixmap pixmap = button->iconSet()->pixmap( QIconSet::Small, mode, state );
			pixw = pixmap.width();

			// If there's no text draw at center, else at right
			// When centered I add one to match tab close icon position
			//   -> why? I do not know, but works
			const int targetY = y + ( h - pixmap.height() )/2;
			if ( button->text().isEmpty() && !button->pixmap() )
				p->drawPixmap( x + ( w - pixw )/2, targetY, pixmap );
			else p->drawPixmap( x + 5, targetY, pixmap );

			ex += pixw + 2;
			ew -= pixw + 2;
		}

		// Does the button have a popup menu?
		if ( button->isMenuButton() ){
			int dx = pixelMetric( PM_MenuButtonIndicator, widget );
			if ( dx + pixw >= w ) {
				drawPrimitive( PE_ArrowDown, p, visualRect( QRect( x2 - 6, x + h - 6, 7, 7 ), r ),
						cg, flags, opt );
			} else {
				drawPrimitive( PE_ArrowDown, p, visualRect( QRect( x2 - dx - 8, y + 2, dx, h - 4 ), r ),
						cg, flags, opt );
				ew -= dx;
			}
		}

		drawItem( p, QRect( ex, y, ew, h ), AlignCenter|ShowPrefix, button->colorGroup(),
				button->isEnabled(), button->pixmap(), button->text(), -1,
				active ? &button->colorGroup().highlightedText() : &button->colorGroup().buttonText() );

		if ( flags & Style_HasFocus ) 
			drawPrimitive( PE_FocusRect, p, subRect(SR_PushButtonFocusRect, button), 
				       cg, flags, opt );

		break;
	}
//	MENUBAR ITEM
//	------------
	case CE_MenuBarItem: {
		QMenuItem *mi = opt.menuItem();

		const int text_flags =
		 AlignVCenter | AlignHCenter | ShowPrefix |
		 DontClip | SingleLine;

		if ( active && focused ) {
			if ( flags & Style_Down ) {
				p->setBrush( MENU_BRUSH2( cg ) );
				p->setPen( MENU_BRUSH2( cg ) );
				renderRoundRect( p, x, y, w, h + 20 );
				p->setPen( cg.buttonText() );
			} else {
				p->setBrush( cg.button() );
				p->setPen( PEN( cg ) );
				renderRoundRect( p, x, y, w, h );
				p->setPen( cg.buttonText() );
			}
		}
		p->drawText( r, text_flags, mi->text() );
		break;
	}

//	POPUPMENU ITEM ( highlighted on mouseover )
//	------------------------------------------
	case CE_PopupMenuItem: {
		const QPopupMenu *pum = ( const QPopupMenu * ) widget;
		static const int itemHMargin = 3;
		static const int itemVMargin = 3;
		static const int arrowHMargin = 6;
		static const int rightBorder = 12;
		const int checkcol = QMAX( opt.maxIconWidth(), 26 );
		const int xm = checkcol + itemHMargin;
		const int tab = opt.tabWidth();


		QMenuItem *mi = opt.menuItem();
		if ( !mi ) return;

		bool active = flags & Style_Active;
		const bool enabled = mi->isEnabled();
		const bool checkable = pum->isCheckable();
		const bool checked = mi->isChecked();

		int xpos = x;
		QColor highlightColor = pum->palette().active().color( QColorGroup::Highlight );
		
		if ( pum->erasePixmap() && !pum->erasePixmap()->isNull() )
			p->drawPixmap( x, y, *pum->erasePixmap(), x, y, w, h );
		else {
			p->fillRect( x, y, w, h, cg.background() );
			p->fillRect( x, y, 22, h, MENU_BRUSH2( cg ) );
		}
		
		if ( mi->isSeparator() ) {
			p->setPen( QPen( MENU_BRUSH2( cg ), 0, DotLine ) );
			p->drawLine( x + 22 + 12, y, x2 - 12, y );
			return;
		}

		p->save();

		if ( active && enabled ){
			//TODO: make this code nicer
			QColor inColor( (int)( cg.base().red()*0.7 + cg.highlight().red()*0.3 ),
					(int)( cg.base().green()*0.7 + cg.highlight().green()*0.3 ),
					(int)( cg.base().blue()*0.7 + cg.highlight().blue()*0.3 ) );
			QColor inColor2( (int)( MENU_BRUSH2( cg ).red()*0.7 + cg.highlight().red()*0.3 ),
					  (int)( MENU_BRUSH2( cg ).green()*0.7 + cg.highlight().green()*0.3 ),
					  (int)( MENU_BRUSH2( cg ).blue()*0.7 + cg.highlight().blue()*0.3 ) );
			p->fillRect( x + 22, y, w - 22 - ROUNDCORNERD/2, h, cg.highlight() );
			p->setPen( highlightColor );
			p->setBrush( inColor );
			renderRoundRect( p, x, y , w, h );
			p->setClipRect( x, y, 22, h);
			p->setClipping(true);
			p->setBrush( inColor2 );
			renderRoundRect( p, x, y, 22 + ROUNDCORNERD, h );
			p->setClipping( false );
		}


		if ( mi->iconSet() ) {
			QIconSet::Mode mode;
			if ( !enabled ) mode = QIconSet::Disabled;
			else if ( checked ) {
				mode = QIconSet::Active;
				p->fillRect( x, y, 22, h, highlightColor.light() );
			} else mode = QIconSet::Normal;
			QPixmap pixmap = mi->iconSet()->pixmap( QIconSet::Small, mode );
			const int pixw = pixmap.width();
			const int pixh = pixmap.height();
			p->drawPixmap( x + ( 22 - pixw )/2, y + ( h - pixh )/2, pixmap );
		} else if ( checkable && checked ) {
			SFlags cflags = Style_Default | Style_On;
			if ( enabled ) cflags |= Style_Enabled;
			QRect rr = QRect( x + 3, y + 3, 16, h - 5 );
			drawPrimitive( PE_CheckMark, p, rr, cg, cflags );
		}

		p->setPen( active ? cg.highlightedText() : cg.buttonText() );
		xpos += xm;

		if ( mi->custom() ) {
			int m = itemVMargin;
			//p->save();
			p->setPen( cg.text() );
			mi->custom()->paint( p, cg, active, enabled, x+xm, y+m, w-xm-tab+1, h-2*m );
			p->restore();
			return;
		} else {
			QString s = mi->text();
			if( !s.isNull() ) {
				int t = s.find( '\t' );
				int m = itemVMargin;
				int text_flags = AlignVCenter | ShowPrefix | DontClip | SingleLine;
				
				if ( enabled ) p->setPen( cg.text() );
				else p->setPen( cg.mid().light( 110 ) );

				if ( t >= 0 ) {
					int xp = x + w - tab - rightBorder - itemHMargin + 1;
					QRect rr = QRect( xp, y+m, tab, h-2*m );
					p->drawText( rr, text_flags, s.mid( t+1 ) );
					s = s.left( t );
				}

				QRect rr = QRect( xpos, y+m, w-xm-tab+1, h-2*m );
				p->drawText( rr, text_flags, s );
			} else if ( mi->pixmap() ) {
				QPixmap *pixmap = mi->pixmap();
				if ( pixmap->depth() == 1 ) {
					p->setBackgroundMode( OpaqueMode );
				}
				p->drawPixmap( xpos, y, *pixmap );
				if ( pixmap->depth() == 1 ) {
					p->setBackgroundMode( TransparentMode );
				}
			}
		}

		if ( mi->popup() && enabled ) {
			int dim = pixelMetric( PM_MenuButtonIndicator );
			xpos = x+w - arrowHMargin - dim;
			QRect rr = QRect( xpos, y + h/2 - dim/2, dim, dim );
			drawPrimitive( PE_ArrowRight, p, rr, cg, flags );
		}

		p->restore();
		break;
	}
	case CE_RadioButton: {
		// Taken the look from konx
		int min = QMIN( w, h );
		p->setBrush( cg.base() );
		p->setPen( PEN( cg ) );
		p->drawEllipse( x + 1, y + 1, min - 2, min - 2 );

		if ( flags & Style_On || flags & Style_Down ) {
			p->setPen( flags & Style_Down ? radioColor.light() :
					flags & Style_Enabled ? radioColor : radioColor.dark() );
			p->setBrush( flags & Style_Down ? radioColor.light() :
					flags & Style_Enabled ? radioColor : radioColor.dark() );
			p->drawEllipse( x + 4, y + 4, min - 8, min - 8 );
		}
		break;
	}
	case CE_CheckBox:
	{
		// Taken the look from konx
		p->setPen( PEN( cg ) );
		p->setBrush( flags & Style_Enabled ? cg.base() : cg.background() );
		p->drawRect( r );
		p->setPen( cg.foreground() );
		drawPrimitive( PE_CheckMark, p, r, cg, flags );
		break;
	}

	//	PROGRESSBAR: 
	//	-----------
	case CE_ProgressBarGroove: {
		p->setPen( PEN( cg ) );
		p->setBrush( flags & Style_Enabled? cg.base(): cg.background() );
		renderRoundRect( p, x, y, w, h );
		break;
	}

	// From konx
	case CE_ProgressBarContents: {
		const QProgressBar *pb = ( const QProgressBar * )widget;
		const int progress = pb->progress();
		int xLine = x + 10;
		const int totalSteps = pb->totalSteps();
		//kdDebug() << progress << "/" << totalSteps << "\n";
		if ( ( totalSteps > 0 ) && ( progress > 0 ) ) {
			const int progressPx = ( progress * w ) / totalSteps;
			p->setBrush( cg.highlight() );
			p->setPen( PEN( cg ) );
			// Correction for too small content
			if ( progressPx < 10 ) {
				p->setClipRect( x, y, progressPx, h );
				p->setClipping( true );
				renderRoundRect( p, x, y, progressPx + ROUNDCORNERD, h );
				p->setClipping( false );
				p->drawLine( progressPx, y, progressPx, y + h-1 );
			} else renderRoundRect( p, x, y, progressPx, h );
			
			// draw some small vertical lines
			p->setPen( cg.highlight().light() );
			// We take care the lines do not overwrite the content border
			const int xLineEnd = QMIN( x + progressPx-1, w-5 );
			if ( h >= 10 ) for ( ; xLine < xLineEnd; xLine += 10 )
					p->drawLine( xLine, y + 5, xLine, h - 6 );
			if ( xLine-1 == progressPx ) xLine +=10;

		}
		
		p->setPen( cg.mid() );
		if ( h >= 10 ) for ( ; xLine < w - 5; xLine += 10 )
				p->drawLine( xLine, y + 5, xLine, h - 6 );
		
		break;
	}
	default: KStyle::drawControl( element, p, widget, r, cg, flags, opt );
	}
}



void dotCURVEstyle::drawComplexControl( ComplexControl control,
				 QPainter *p,
				 const QWidget *widget,
				 const QRect &r,
				 const QColorGroup &cg,
				 SFlags flags,
				 SCFlags controls,
				 SCFlags active,
				 const QStyleOption& opt ) const
{
	int x, y, w, h, x2, y2;
	r.rect( &x, &y, &w, &h );
	x2 = x + w -1;
	y2 = w + h -1;
	const bool enabled = flags & Style_Enabled;
	switch( control ) {
	//	LISTVIEW
	//	--------
	case CC_ListView: {
		// Taken from konx
		if ( controls & SC_ListView )
				KStyle::drawComplexControl( control, p, widget, r, cg, flags, controls, active, opt );

		if ( controls & ( SC_ListViewBranch | SC_ListViewExpand ) ) {
			if ( opt.isDefault() )
			break;

			QListViewItem *item = opt.listViewItem();
			QListViewItem *child = item->firstChild();

			int y = r.y();
			int c;
			if ( active == SC_All && controls == SC_ListViewExpand ) c = 2;
			else {
				int linebot = 0;

				// skip the stuff above the exposed rectangle
				while ( child && y + child->height() <= 0 ) {
					y += child->totalHeight();
					child = child->nextSibling();
				}

				int bx = r.width() / 2;

				// paint stuff in the magical area
				QListView* v = item->listView();
				while ( child && y < r.height() ) {
					int lh;
					if ( !item->multiLinesEnabled() )
						lh = child ? child->height() : 0;
					else
						lh = p->fontMetrics().height() + 2 * v->itemMargin();
					lh = QMAX( lh, QApplication::globalStrut().height() );
					if ( lh % 2 > 0 )
						lh++;
					linebot = y + lh/2;

					if ( ( child->isExpandable() || child->childCount() ) && ( child->height() > 0 ) ) {
						p->setPen( cg.dark() );
						// Closed
						int s;
						if ( child->isOpen() ) // open
							s = DOWN_ARROW;
						else //or closed
							s = RIGHT_ARROW;

						arrowPoints[s].translate( bx, linebot );
						p->drawLineSegments( arrowPoints[s], 0, 3 );
						p->drawPoint( arrowPoints[s][6] );
						// Restore to original posotion
						arrowPoints[s].translate( -bx, -linebot );
					}
					y += child->totalHeight();
					child = child->nextSibling();
				}
			}
			p->setPen( cg.text() );
		}
		break;
	}

	//	COMBOBOX
	//	--------
	case CC_ComboBox: {
		const QComboBox *cb = ( const QComboBox * )widget;
		//cleanSides( p, x, y, w, h, cg.background() );
		static const unsigned int handle_width = 16;
		static const unsigned int handle_offset = handle_width + 1;


		p->setPen( PEN( cg ) );
		
		if ( flags & Style_Enabled ) p->setBrush( cg.base() );
		else p->setBrush( cg.background() );
		if ( !khtml ) cleanSides( p, x, y, w, h, cg.background() );
		renderRoundRect( p, x, y, w, h );
		
		// HACK: 15 is from eye-o-meter
		const int offset=15;
		p->setClipping( true );
		p->setClipRect( x2 - offset, y, offset, h );
		p->setBrush( cg.button() );
		renderRoundRect( p, x2 - offset - ROUNDCORNERD+1, y, offset + ROUNDCORNERD, h );
		p->setClipping( false );

		// draw the arrow depending whether the combo box is opened or not
		bool opened = cb->listBox() ? cb->listBox()->isVisible() : false;

		QRect cr( w - handle_offset + 1, ( opened ? y + 1 : y ), handle_width, h - 2 );
		QRect pmr( 0, 0, 7, 4 );
		pmr.moveCenter( cr.center() );

		drawPrimitive( ( opened ? PE_ArrowDown : PE_ArrowLeft ), p, pmr, cg, Style_Enabled );

		// draw focus rectangle
		if ( flags & Style_HasFocus )
		{
			QRect focusRect = visualRect( subRect( SR_ComboBoxFocusRect, widget ), widget );
			drawPrimitive( PE_FocusRect, p, focusRect, cg, flags );
		}

		break;
	}

//	TOOLBUTTON
//	----------
	case CC_ToolButton: {
		const QToolButton *tb = ( const QToolButton * ) widget;
		
		QRect button, menuarea;
		button = querySubControlMetrics( control, widget, SC_ToolButton, opt );
		menuarea = querySubControlMetrics( control, widget, SC_ToolButtonMenu, opt );

		// HACK: The more ugly hack I've ever done. If you quit this code the 
		// arrow buttons for tabbars does not draw the right border and 
		// overlap the page when is below. This is because PM_DefaultFrameWidth
		// is 3. When is <3 this doesn't happen. Why?
		if ( tb->parentWidget() && tb->parentWidget()->inherits("QTabBar") ){
			bool below = false;
			const QTabBar *tabBar = static_cast<const QTabBar*>( tb->parentWidget() );
			switch ( tabBar->shape() ) {
			case QTabBar::RoundedBelow:
			case QTabBar::TriangularBelow:
				++( button.rTop() );
				below = true;
				break;
			default: /* To stop g++ complaints */
				break;
			}
			if ( !strcmp( widget->name(), "qt_right_btn" ) ) {
				button.moveBy( -1, 0 );
				if ( below ) p->drawPoint( x2 -1, y );
			}
		}
		
		SFlags bflags = flags,
		 mflags = flags;

		if ( active & SC_ToolButton )
			bflags |= Style_Down;

		if ( active & SC_ToolButtonMenu )
			mflags |= Style_Down;

		if ( controls & SC_ToolButton ) {
		//	If we're pressed, on, or raised...
			if ( bflags & ( Style_Down | Style_On | Style_Raised ) ) {
				drawPrimitive( PE_ButtonTool, p, button, cg, bflags, opt );
			} else if ( tb->parentWidget() &&
				 tb->parentWidget()->backgroundPixmap() &&
				 !tb->parentWidget()->backgroundPixmap()->isNull() ) {
				QPixmap pixmap = *( tb->parentWidget()->backgroundPixmap() );
				p->drawTiledPixmap( r, pixmap, tb->pos() );
			}
		}

		// Draw a toolbutton menu indicator if required
		if ( controls & SC_ToolButtonMenu ) {
			if ( mflags & ( Style_Down | Style_On | Style_Raised ) ) {
				drawPrimitive( PE_ButtonDropDown, p, menuarea, cg, mflags, opt );
			}
			drawPrimitive( PE_ArrowDown, p, menuarea, cg, mflags, opt );
		}

		if ( tb->hasFocus() && !tb->focusProxy() ) {
			QRect fr = tb->rect();
			fr.addCoords( 2, 2, -2, -2 );
			drawPrimitive( PE_FocusRect, p, fr, cg );
		}

		break;
	}

//	SPINWIDGETS
//	-----------
	case CC_SpinWidget: {
		const QSpinWidget *sw = ( const QSpinWidget * ) widget;
		SFlags flags;
		PrimitiveElement pe;

		if ( controls & SC_SpinWidgetFrame ){
			p->setBrush( enabled? cg.base() : cg.background() );
			p->setPen( PEN( cg ) );
			renderRoundRect( p, x, y, w, h );
			const int offset = sw->upRect().width();
			p->setClipping( true );
			p->setClipRect( x2 - offset, y, offset, h );
			p->setBrush( cg.button() );
			renderRoundRect( p, x2 - offset - ROUNDCORNERD, y, offset + ROUNDCORNERD + 1, h );
			p->setClipping( false );
		}

		if ( controls & SC_SpinWidgetUp ) {
			flags = Style_Default | Style_Enabled;
			if ( active == SC_SpinWidgetUp ) {
				flags |= Style_On;
				flags |= Style_Sunken;
			} else
				flags |= Style_Raised;

			pe = sw->buttonSymbols() == QSpinWidget::PlusMinus ?
				pe = PE_SpinWidgetPlus : pe = PE_SpinWidgetUp;

			QRect re = sw->upRect();
			re.rLeft() += 3;
			QColorGroup ucg = sw->isUpEnabled() ? cg : sw->palette().disabled();
			drawPrimitive( pe, p, re, ucg, flags );
		}

		if ( controls & SC_SpinWidgetDown ) {
			flags = Style_Default | Style_Enabled;
			if ( active == SC_SpinWidgetDown ) {
				flags |= Style_On;
				flags |= Style_Sunken;
			} else flags |= Style_Raised;

			pe = sw->buttonSymbols() == QSpinWidget::PlusMinus ?
				pe = PE_SpinWidgetMinus : PE_SpinWidgetDown;

			QRect re = sw->downRect();
			re.rLeft() += 3;
			QColorGroup dcg = sw->isDownEnabled() ? cg : sw->palette().disabled();
			drawPrimitive( pe, p, re, dcg, flags );
		}

		break;
	}
	default:
		KStyle::drawComplexControl( control, p, widget,
					 r, cg, flags, controls,
					 active, opt );
		break;
	}
}


QRect dotCURVEstyle::subRect( SubRect r, const QWidget *widget ) const
{
	QRect qrect;
	switch ( r ) {
	case SR_ProgressBarContents: 
	case SR_CheckBoxFocusRect:
	case SR_RadioButtonFocusRect:
	case SR_PushButtonContents:
		if ( !widget->inherits( "KColorButton" ) ) return widget->rect();
	case SR_PushButtonFocusRect:
		qrect = QRect( 4, 3, widget->width() - 8, widget->height() - 6 );
		break;
	case SR_CheckBoxIndicator:
		qrect = KStyle::subRect( r, widget );
		if ( !khtml ) qrect.moveBy( 2, 0 );
		break;
	default:
		qrect = KStyle::subRect( r, widget );
	}
	return qrect;
}


int dotCURVEstyle::pixelMetric( PixelMetric m, const QWidget *widget ) const
{
 
	switch( m ) {
	// TABS
	// ----
	
	case PM_TabBarTabVSpace: {
		const QTabBar * tb = (const QTabBar *) widget;
		if (   tb->shape() == QTabBar::RoundedAbove
		    || tb->shape() == QTabBar::RoundedBelow )
			return 12;
		else
			return 4;
	}
	
	case PM_TabBarScrollButtonWidth:return 15;
	
	case PM_TabBarTabOverlap: return 1;
	case PM_TabBarBaseOverlap: return 0;
	case PM_TabBarBaseHeight: return 0;
	
	// PROGRESSBARS
	// ------------
	case PM_ProgressBarChunkWidth: return 9;

	// SLIDER
	// ------
	case PM_SliderLength: return 12;

	case PM_DefaultFrameWidth:
	// HACK: For my loved Konqueror
		if (   widget 
	 	    && widget->parentWidget( true )
		    && widget->parentWidget( true )->inherits( "KonqFrame" ) )
			return 0;
	case PM_SpinBoxFrameWidth:
		return 3;
	// CHECKBOXES / RADIO BUTTONS
	// --------------------------
	case PM_ExclusiveIndicatorWidth: // Radiobutton size
	case PM_ExclusiveIndicatorHeight: // 13x13
	case PM_IndicatorWidth:	 // Checkbox size
	case PM_IndicatorHeight:	 // 13x13
		return 13;

	// MENU INDICATOR AND BUTTONS
	// --------------------------
	case PM_MenuButtonIndicator:
	case PM_ButtonMargin:
	case PM_ButtonShiftHorizontal:
	case PM_ButtonShiftVertical:
	case PM_ButtonDefaultIndicator: 
		return 0;

	// From plastik:
#if (QT_VERSION >= 0x030300) // requires Qt 3.3
    	// extra space between menubar items
        case PM_MenuBarItemSpacing: {
	return 6;
	}
#endif		
	default:
		return KStyle::pixelMetric( m, widget );
	}
}


QSize dotCURVEstyle::sizeFromContents( ContentsType t,
				 const QWidget *widget,
				 const QSize &s,
				 const QStyleOption &opt ) const
{
//	int w = s.width(), h = s.height();
	switch ( t ) {
	case CT_TabBarTab: {
		QSize size = KStyle::sizeFromContents( t, widget, s, opt );
		size.rheight() += 2;
		return size;
	}

	case CT_PopupMenuItem: {
		QMenuItem *mi = opt.menuItem();
		int w = s.width(), h = s.height();;
		if ( !widget || opt.isDefault() )
			break;

		if ( mi->custom() ) return mi->custom()->sizeHint();

		if ( mi->widget() ) { /* don't change the size in this case. */ }
		else if ( mi->isSeparator() ) {
			w = 10;
			h = 1;
		} else {
			w += 50;
			h += 4;
			if ( mi->popup() ) w += 20;
		}
		
		return QSize( w, h );
	}
	case CT_PushButton: {
		// Taken ( stolen ) from plastik
		const QPushButton* btn = static_cast<const QPushButton*>( widget );

		int w = s.width() + 8;
		int h = s.height() + 8;
		if ( btn->text().isEmpty() && s.width() < 32 ) return QSize( w, h );

		return QSize( w + 15, h +1 );
	}
	// to stop gcc complaints
	default: break;
	}
	return KStyle::sizeFromContents( t, widget, s, opt );
}

void dotCURVEstyle::polish( QWidget* widget ){

	/*
	kdDebug() << "--\n";
	QWidget *w;
	for ( w = widget; w; w = w->parentWidget() )
		kdDebug() << w << ": " << w->paletteBackgroundColor().name() << "\n";
	*/
	if( !strcmp( widget->name(), "__khtml" ) ) 
		khtml = true;
	
	if ( ::qt_cast<QPopupMenu*>( widget ) )
		widget->setBackgroundMode( NoBackground );
	else if ( ::qt_cast<QHeader*>( widget ) )
		widget->setBackgroundMode( PaletteBase );
	else if ( widget->backgroundMode() == PaletteButton )
		widget->setBackgroundMode( PaletteBackground );
	 else if( ::qt_cast<QLineEdit*>( widget ) ) {
		widget->setBackgroundMode( X11ParentRelative );
	 }
/*	if ( kickerMode && widget->inherits( "QHeader" ) )
		widget->setBackgroundMode( NoBackground );
*/
	KStyle::polish( widget );
}

void dotCURVEstyle::polish( QApplication* app )
{
	if ( !qstrcmp( app->argv()[0], "kicker" ) ){
//		kdDebug() << "kicker!\n";
		kickerMode = true;
	} else kickerMode=false;
}


// void dotCURVEstyle::unPolish( QWidget* widget )


int dotCURVEstyle::styleHint( StyleHint stylehint,
			 const QWidget * widget,
			 const QStyleOption & opt,
			 QStyleHintReturn * returnData ) const
{
	switch ( stylehint ) {
	case SH_TabBar_Alignment: return AlignLeft;
	case SH_Header_ArrowAlignment: return Right;
	// to stop gcc complaints
	default: break;
	}
	return KStyle::styleHint( stylehint, widget, opt, returnData );
}

